@@ -45,11 +45,15 @@
 #' }
 #' }
 #'
-#' @inherit as_pkg_ref params return
+#' @section Package Cohorts:
+#'
+#' *Experimental!*
+#' Package cohorts are structures to determine the risk of a set of packages.
+#' `pkg_library()` can be called to create a object containing the pkg_ref
+#' objects of all packages in a system library.
 #'
 #'
-#'
-#' @family pkg_ref
+#' @rdname pkg_ref
 #' @export
 #'
 #' @examples
@@ -80,21 +84,130 @@ new_pkg_ref <- function(name, version = NA_character_, source, ...) {
 
   source <- match.arg(
     source,
-    c("pkg_bioc_remote", "pkg_cran_remote", "pkg_remote", "pkg_install", "pkg_source", "pkg_missing"),
+    c("pkg_git_remote", "pkg_bioc_remote", "pkg_cran_remote", "pkg_remote", "pkg_install", "pkg_source", "pkg_missing"),
     several.ok = TRUE)
 
+  source <- get_pkg_ref_classes(source)
+
   pkg_data <- as.environment(append(list(
     name = name,
     version = version,
-    source = source
+    source = source[[1L]]
   ), dots))
 
   structure(
     pkg_data,
-    class = c(source, "pkg_ref", class(pkg_data)))
+    class = c(source, class(pkg_data)))
 }
 
 
+#' The `pkg_ref` subclass hierarchy, used for pkg_ref object creation with a
+#' specified subclass
+#'
+pkg_ref_class_hierarchy <- list(
+  "pkg_ref" = list(
+    "pkg_missing",
+    "pkg_source",
+    "pkg_install",
+    "pkg_remote" = list(
+      "pkg_cran_remote",
+      "pkg_bioc_remote",
+      "pkg_git_remote"
+    )
+  )
+)
+
+
+#' Walk the pkg_ref class hierarchy to match a single subclass to a class path
+#'
+#' @param x (`character(1L)`) A subclass, among those known in pkg_ref subclasses
+#' @param classes (`list`) A class hierarchy, described using a named list.
+#'   Defaults to `pkg_ref_class_hierarchy`.
+#'
+#' @return A `character(n)` class path from `pkg_ref` down to the specified
+#'   subclass, or `FALSE` if no path is found.
+#'
+get_pkg_ref_classes <- function(x, classes = pkg_ref_class_hierarchy) {
+  if (x %in% names(classes) || x %in% classes) 
+    return(x)
+
+  if (!is.list(classes))
+    return(FALSE)
+
+  for (i in seq_along(classes)) {
+    subclasses <- get_pkg_ref_classes(x, classes[[i]])
+    if (is.character(subclasses)) return(c(subclasses, names(classes[i])))
+  }
+
+  FALSE  
+}
+
+
+#' @rdname pkg_ref
+#'
+#' @param lib.loc The path to the R library directory of the installed package.
+pkg_install <- function(x, lib.loc = NULL) {
+  if(verify_pkg_source(x, "pkg_install") == "pkg_missing") return(pkg_missing(x))
+
+  path <- find.package(x, lib.loc = lib.loc)
+  version <- utils::packageVersion(x, lib.loc = dirname(path))
+
+  new_pkg_ref(x,
+              version = version,
+              path = path,
+              source = "pkg_install")
+}
+
+#' @rdname pkg_ref
+pkg_source <- function(x) {
+  desc <- read.dcf(file.path(x, "DESCRIPTION"))
+  name <- unname(desc[,"Package"])
+
+  new_pkg_ref(name,
+              version = package_version(desc[,"Version"][[1]]),
+              path = normalizePath(x),
+              source = "pkg_source")
+}
+
+#' @rdname pkg_ref
+#'
+#' @param repos URL of CRAN repository to pull package metadata.
+pkg_cran <- function(x, repos = getOption("repos", "https://cran.rstudio.com")) {
+  ap <- memoise_available_packages(repos = repos)
+  info <- ap[ap[,"Package"] == x,,drop = FALSE]
+
+  new_pkg_ref(x,
+              version = info[,"Version"],
+              repo = info[,"Repository"],
+              source = c("pkg_cran_remote"))
+}
+
+#' @rdname pkg_ref
+pkg_bioc <- function(x) {
+  bp <- memoise_bioc_available()
+  info <- bp[bp[,"Package"] == x,,drop = FALSE]
+
+  new_pkg_ref(x,
+              version = info[,"Version"],
+              repo = "https://bioconductor.org/packages/release/bioc",
+              source = c("pkg_bioc_remote"))
+}
+
+#' @rdname pkg_ref
+pkg_missing <- function(x) {
+  new_pkg_ref(x,
+              source = c("pkg_missing"))
+}
+
+#' @rdname pkg_ref
+pkg_library <- function(lib.loc) {
+  # Create pkg_cohort object
+  cohort <- pkg_cohort()
+  for(pkg in list.files(lib.loc, recursive = FALSE, full.names = FALSE)) {
+    cohort[[length(cohort)+1]] <- pkg_install(pkg, lib.loc = lib.loc)
+  }
+  cohort
+}
 
 #' Convert into a package object
 #'
@@ -110,7 +223,7 @@ new_pkg_ref <- function(name, version = NA_character_, source, ...) {
 #'   considered analogous to a \code{list}. See 'Details' for further
 #'   information about \code{pkg_ref} subclasses.
 #'
-#' @family pkg_ref
+#' @rdname pkg_ref
 #'
 #' @importFrom vctrs new_list_of
 #' @export
@@ -152,7 +265,6 @@ as_pkg_ref.pkg_ref <- function(x, ...) {
 }
 
 
-
 #' @importFrom utils installed.packages available.packages packageVersion
 #' @export
 as_pkg_ref.character <- function(x, repos = getOption("repos", "https://cran.rstudio.com"),
@@ -168,47 +280,12 @@ as_pkg_ref.character <- function(x, repos = getOption("repos", "https://cran.rst
                                "pkg_bioc_remote", "pkg_missing"))
 
   switch(pkg_source_,
-         pkg_install = {
-           path <- find.package(x, lib.loc = lib.loc)
-           version <- utils::packageVersion(x, lib.loc = dirname(path))
-
-           new_pkg_ref(x,
-                       version = version,
-                       path = path,
-                       source = "pkg_install")
-         },
-         pkg_source = {
-
-           desc <- read.dcf(file.path(x, "DESCRIPTION"))
-           name <- unname(desc[,"Package"])
-
-           new_pkg_ref(name,
-                       version = package_version(desc[,"Version"][[1]]),
-                       path = normalizePath(x),
-                       source = "pkg_source")
-         },
-         pkg_cran_remote = {
-           ap <- memoise_available_packages(repos = repos)
-           info <- ap[ap[,"Package"] == x,,drop = FALSE]
-
-           new_pkg_ref(x,
-                       version = info[,"Version"],
-                       repo = info[,"Repository"],
-                       source = c("pkg_cran_remote"))
-         },
-         pkg_bioc_remote = {
-           bp <- memoise_bioc_available()
-           info <- bp[bp[,"Package"] == x,,drop = FALSE]
-
-           new_pkg_ref(x,
-                       version = info[,"Version"],
-                       repo = "https://bioconductor.org/packages/release/bioc",
-                       source = c("pkg_bioc_remote"))
-         },
-         pkg_missing = {
-           new_pkg_ref(x,
-                       source = c("pkg_missing"))
-         })
+         pkg_install = pkg_install(x, lib.loc = lib.loc),
+         pkg_source = pkg_source(x),
+         pkg_cran_remote = pkg_cran(x, repos = repos),
+         pkg_bioc_remote = pkg_bioc(x),
+         pkg_missing = pkg_missing(x)
+  )
 }
 
 #' Determine the intended source of a new package
@@ -306,18 +383,3 @@ verify_pkg_source <- function(x, source, repos) {
 
   source
 }
-
-is_available_cran <- function(x, repos, p) {
-  x %in% memoise_available_packages(repos = repos)[,"Package"] ||
-    (!is.null(memoise_cran_mirrors()) &&
-       # isTRUE added to catch any issues where the cran mirror isn't available
-       isTRUE(is_url_subpath_of(
-         p$repo_base_url,
-         c(memoise_cran_mirrors()$URL, "https://cran.rstudio.com/"))))
-}
-
-is_available_bioc <- function(x, p){
-  x %in% memoise_bioc_available()[,"Package"] ||
-    (!is.null(memoise_bioc_mirrors()) &&
-       isTRUE(is_url_subpath_of(p$repo_base_url, memoise_bioc_mirrors()$URL)))
-}
